local compi = {}
local locations = require(mod_name .. ".lualib.locations")
local command_callback = require(mod_name .. ".lualib.command_callback")
local function_save = require(mod_name .. ".lualib.function_save")
local util = require("util")
local math2d = require("math2d")

local walk_to_internal

compi.set_direction = function(direction)
  direction = direction or defines.direction.southeast

  local compi_ent = compi.get_entity()

  local deg45 = 1/8;

  if direction == defines.direction.north then
    compi_ent.orientation = deg45 * 0
  elseif direction == defines.direction.northeast then
    compi_ent.orientation = deg45 * 1
  elseif direction == defines.direction.east then
    compi_ent.orientation = deg45 * 2
  elseif direction == defines.direction.southeast then
    compi_ent.orientation = deg45 * 3
  elseif direction == defines.direction.south then
    compi_ent.orientation = deg45 * 4
  elseif direction == defines.direction.southwest then
    compi_ent.orientation = deg45 * 5
  elseif direction == defines.direction.west then
    compi_ent.orientation = deg45 * 6
  elseif direction == defines.direction.northwest then
    compi_ent.orientation = deg45 * 7
  end

  global.compilatron.direction = direction
end

compi.ensure_spawned = function()
  if not (global.compilatron.entity and global.compilatron.entity.valid) then
    compi.respawn_compi()
  end

  assert(global.compilatron.entity and global.compilatron.entity.valid)
end

compi.get_entity = function(ensure)
  if ensure == nil then
    ensure = true
  end

  if ensure then
    compi.ensure_spawned()
  end

  return global.compilatron.entity
end

compi.stop = function()
  compi.get_entity().set_command({ type=defines.command.stop })
end

compi.moving = function()
  return compi.get_entity().moving
end

compi.teleport_to = function (position)
  compi.get_entity().teleport(position)
end

compi.walk_to = function(_destination, teleport_on_fail, allow_destroy_friendly_entities, arrived_callback_name)
  compi.remove_display()

  if teleport_on_fail == nil then teleport_on_fail = true end
  if allow_destroy_friendly_entities == nil then allow_destroy_friendly_entities = true end

  local destroy_immediately = false
  if allow_destroy_friendly_entities == "immediately" then
    allow_destroy_friendly_entities = true
    destroy_immediately = true
  end

  local destination
  if _destination.x == nil then
    destination = {_destination[1], _destination[2]}
  else
    destination = {_destination.x, _destination.y}
  end

  destination[1] = math.floor(destination[1]) + 0.5
  destination[2] = math.floor(destination[2]) + 0.5

  global.compilatron.walk_to_retry_data =
  {
    destination = destination,
    teleport_on_fail = teleport_on_fail,
    allow_destroy_friendly_entities = allow_destroy_friendly_entities,
    arrived_callback = arrived_callback_name,
  }

  walk_to_internal(
    destination,
    teleport_on_fail,
    destroy_immediately,
    false,
    "walk_to_retry",
    arrived_callback_name)
end

-- target: entity or bounding_box to walk near to
-- do_indicate: whether to draw an indication box around the target
-- say: what to say when we get there
compi.stand_next_to = function(target, do_indicate, say, arrived_callback_name, allow_destroy_friendly_entities)
  if allow_destroy_friendly_entities == nil then
    allow_destroy_friendly_entities = true
  end

  assert(target, "attempted to tell Compi to stand_next_to nil target")
  local is_bounding_box = false
  for k,_ in pairs(target) do
    if k == "left_top" then
      is_bounding_box = true
      break
    end
  end
  local bounding_box = target
  if is_bounding_box == false then
    bounding_box = target.bounding_box
  end
  local start_bounding_box = bounding_box

  bounding_box = {
    left_top = {
      x = bounding_box.left_top.x - 2,
      y = bounding_box.left_top.y - 2,
    },
    right_bottom = {
      x = bounding_box.left_top.x,
      y = bounding_box.left_top.y,
    },
  }

  local position = locations.get_main_surface().find_non_colliding_position_in_box('compilatron',bounding_box,0.01,true)
  local tries = 0
  while position == nil and tries < 4 do
    bounding_box = {
      left_top = {
        x = bounding_box.left_top.x,
        y = bounding_box.left_top.y,
      },
      right_bottom = {
        x = bounding_box.right_bottom.x + 3,
        y = bounding_box.right_bottom.y + 3,
      },
    }
    position = locations.get_main_surface().find_non_colliding_position_in_box('compilatron',bounding_box,0.01,true)
    if math2d.bounding_box.contains_point(start_bounding_box, position) then
      position = nil
    end
    tries = tries + 1
  end

  if position then
    compi.walk_to(position, true, allow_destroy_friendly_entities, arrived_callback_name)

    if do_indicate then
      global.compilatron.future_box = target
    end

    if say then
      compi.say_later(say)
    end
  else
    global.compilatron.reset = true
    print("Compi was blocked")
  end
end

compi.say = function(text, lifetime)
  if global.compilatron.bubble and global.compilatron.bubble.valid then
    global.compilatron.bubble.start_fading_out()
  end

  if text == nil then
    global.compilatron.wait_text = nil
  end

  global.compilatron.now_text = text
  global.compilatron.now_text_lifetime = lifetime
  global.compilatron.last_text = text
end

function compi.remove_display()
  if global.compilatron.indicator then
    global.compilatron.indicator.destroy() global.compilatron.indicator = nil
  end
  if global.compilatron.future_box then
    global.compilatron.future_box = nil
  end
  if global.compilatron.bubble and global.compilatron.bubble.valid then
    global.compilatron.bubble.start_fading_out()
  end
end

function compi.say_later(text)
  compi.say()
  global.compilatron.wait_text = text
end

function compi.indicate(target)
  if target == nil then
    compi.remove_display()
    global.compilatron.future_box = nil
    return
  end
  if global.compilatron.indicator then global.compilatron.indicator.destroy() global.compilatron.indicator = nil end

  local is_bounding_box = false
  for k,_ in pairs(target) do
    if k == "left_top" then
      is_bounding_box = true
      break
    end
  end

  if is_bounding_box then
    global.compilatron.indicator = compi.get_entity().surface.create_entity({
      name='highlight-box',
      bounding_box = target,
      position = {0,0}
    })
  else -- target is an entity
    if target.valid then
      global.compilatron.indicator = compi.get_entity().surface.create_entity({
        name='highlight-box',
        source = target,
        position = {0,0}
      })
    end
  end
  --print("boop:",serpent.block(box))
  global.compilatron.future_box = nil
end

function compi.check_range(position,range)
  local dist = math2d.position.distance(compi.get_entity().position,position)
  return dist < range
end

compi.respawn_compi = function(spawn_pos)
  assert(global.compilatron.entity == nil or (not global.compilatron.entity.valid))

  spawn_pos = spawn_pos or global.compilatron.wait_position or {0,0}
  local new_compi = locations.get_main_surface().create_entity
    {
      name = "compilatron",
      position = spawn_pos,
      force = "player"
    }
  new_compi.set_command { type=defines.command.stop }
  new_compi.destructible = false

  global.compilatron.entity = new_compi
  return new_compi
end

compi.build_area = function(area_name, search_params, speed)
  assert(not global.compilatron_build_data)
  if campaign_debug_mode then
    print("Compi start building")
  end

  speed = speed or 1

  search_params = util.table.deepcopy(search_params)
  search_params.area = locations.get_area(area_name)

  local entities_to_build_in_template_surface = locations.get_template_surface().find_entities_filtered(search_params)

  local compi_roboport = locations.get_main_surface().create_entity
  {
    name = "compi-roboport",
    position = compi.get_entity().position,
    force = "compi-build"
  }

  local construction_robot_count = math.ceil(#entities_to_build_in_template_surface / 5)

  compi_roboport.insert{ name = "construction-robot", count = construction_robot_count}

  local compi_logistics_chest = locations.get_main_surface().create_entity
  {
    name = "compi-logistics-chest",
    position = compi.get_entity().position,
    force = "compi-build"
  }

  global.compilatron_build_data =
  {
    roboport = compi_roboport,
    chest = compi_logistics_chest,
    build_robot_count = construction_robot_count,
    build_targets = {}
  }

  local force = game.forces['compi-build']
  force.reset()
  force.worker_robots_speed_modifier = speed

  for _, entity in pairs(entities_to_build_in_template_surface) do

    if entity.prototype.items_to_place_this then

      -- Remove everything in the way
      local entity_width = entity.prototype.collision_box.right_bottom.x - entity.prototype.collision_box.left_top.x
      local entity_height = entity.prototype.collision_box.right_bottom.y - entity.prototype.collision_box.left_top.y

      local masks = {}
      for mask, on in pairs(entity.prototype.collision_mask) do
        if on then
          table.insert(masks, mask)
        end
      end

      local overlapping_entities = locations.get_main_surface().find_entities_filtered
      {
        area = math2d.bounding_box.create_from_centre(entity.position, entity_width, entity_height),
        collision_mask = masks
      }

      for _, overlapping_entity in pairs(overlapping_entities) do
        overlapping_entity.destroy()
      end

      -- And then arrange ghosts/items to build
      compi_logistics_chest.insert
      {
        name = entity.prototype.items_to_place_this[1].name,
        count = 1
      }

      locations.get_main_surface().create_entity
      {
        name = "entity-ghost",
        inner_name = entity.name,
        position = entity.position,
        direction = entity.direction,
        force = "compi-build"
      }

      global.compilatron_build_data.build_targets[entity.name .. serpent.line(entity.position)] = entity
    end
  end

  global.compi_is_building = true
end


---------------
-- Internals --
---------------

local on_robot_built_entity = function(event)
  if event.created_entity.force.name == 'compi-build' then
    event.created_entity.force = 'player'

    if event.created_entity.get_fuel_inventory() then
      event.created_entity.get_fuel_inventory().insert({name='coal',count=2})
    end

    local template = global.compilatron_build_data.build_targets
      [event.created_entity.name .. serpent.line(event.created_entity.position)]
    event.created_entity.operable = template.operable
    event.created_entity.active = template.active
    event.created_entity.rotatable = template.rotatable
    event.created_entity.minable = template.minable
    event.created_entity.destructible = template.destructible

    global.compilatron_build_data.build_targets[event.created_entity.name .. serpent.line(event.created_entity.position)] = nil
  end
end

local check_building_done = function()
  if not global.compilatron_build_data then
    return
  end

  local targets_remaining = 0
  for _, _ in pairs(global.compilatron_build_data.build_targets) do
    targets_remaining = targets_remaining + 1
  end

  local robots_in_roboport = (not global.compilatron_build_data.roboport.valid) or
    global.compilatron_build_data.roboport.get_item_count('construction-robot') == global.compilatron_build_data.build_robot_count

  -- if we are finished building
  if targets_remaining == 0 and robots_in_roboport then

    if campaign_debug_mode then
      print("Compi finished building")
    end

    if global.compilatron_build_data.roboport.valid then
      global.compilatron_build_data.roboport.destroy()
    end
    if global.compilatron_build_data.chest.valid then
      global.compilatron_build_data.chest.destroy()
    end

    if global.compi_build_completed_callback then
      function_save.run(global.compi_build_completed_callback)
    else
      global.compilatron.reset = true
    end
    global.compi_build_completed_callback = nil
    global.compi_is_building = false

    global.compilatron_build_data = nil
  end
end

local function on_entity_died(event)
  if event.cause and
     event.cause == compi.get_entity() and
     event.entity.force == compi.get_entity().force and
     event.entity.prototype.is_building
  then
    event.entity.force = "compi_res_force"
    table.insert(global.compilatron.destroyed_entity_positions, event.entity.position)
  end
  if event.entity == compi.get_entity() then
    compi.respawn_compi()
  end
end

local update_ghost_revival = function()
  -- grab the entity ghosts from stuff we destroyed
  for i=#global.compilatron.destroyed_entity_positions,1,-1 do
    local pos = global.compilatron.destroyed_entity_positions[i]
    local bounding_box = {{pos.x, pos.y}, {pos.x + 0.1, pos.y + 0.1}}
    local ghosts = compi.get_entity().surface.find_entities_filtered{type="entity-ghost", area=bounding_box}

    assert(#ghosts == 1)
    table.insert(global.compilatron.destroyed_entity_ghosts, { ghost = ghosts[1], timer = 0})

    table.remove(global.compilatron.destroyed_entity_positions, i)
  end

  -- try to revive any ghosts created by compilatron
  for i=#global.compilatron.destroyed_entity_ghosts,1,-1 do
    if global.compilatron.destroyed_entity_ghosts[i].timer > 60 * 5 then
      local new_entity

      if global.compilatron.destroyed_entity_ghosts[i].ghost.valid then
        _, new_entity = global.compilatron.destroyed_entity_ghosts[i].ghost.revive()
      end

      if new_entity then
        new_entity.force = "player"
        table.remove(global.compilatron.destroyed_entity_ghosts, i)
      else
        -- if we failed to revive, the ghost is probably blocked, just try again later
        global.compilatron.destroyed_entity_ghosts[i].timer = 0
      end
    else
      if not global.compilatron.destroyed_entity_ghosts[i].ghost.valid then
        table.remove(global.compilatron.destroyed_entity_ghosts, i)
      else
        global.compilatron.destroyed_entity_ghosts[i].timer = global.compilatron.destroyed_entity_ghosts[i].timer + 1
      end
    end
  end
end

local update_speech_bubble = function()
  if global.compilatron.now_text and not (global.compilatron.bubble and global.compilatron.bubble.valid) then
    local entity = compi.get_entity()
    entity.direction = defines.direction.southwest

    global.compilatron.bubble = entity.surface.create_entity
    {
      name="compi-speech-bubble",
      text=global.compilatron.now_text,
      lifetime=global.compilatron.now_text_lifetime,
      position={0,0},
      source=entity
    }

    global.compilatron.now_text = nil
    global.compilatron.now_text_lifetime = nil
  end
end

local update = function()
  update_ghost_revival()
  check_building_done()
  update_speech_bubble()
end

local walk_to_retry = function(_)
  local d = global.compilatron.walk_to_retry_data
  walk_to_internal(d.destination, d.teleport_on_fail, d.allow_destroy_friendly_entities, true, nil, d.arrived_callback)
end
function_save.add_function("walk_to_retry", walk_to_retry)

walk_to_internal = function(destination,
                            teleport_on_fail,
                            allow_destroy_friendly_entities,
                            is_retry,
                            retry_callback,
                            arrived_callback)
  global.compilatron.moving = true
  -- if compi is already on his way somewhere, let him finish then queue the move for after
  if global.compilatron.walk_to_internal_retry_data and not is_retry then
    global.compilatron.next_destination_data =
    {
      retry_callback = retry_callback,
      arrived_callback = arrived_callback,
      destination = destination,
      teleport_on_fail = teleport_on_fail,
      allow_destroy_friendly_entities = allow_destroy_friendly_entities,
    }
    return
  end

  compi.remove_display()

  global.compilatron.walk_to_internal_retry_data =
  {
    retry_callback = retry_callback,
    arrived_callback = arrived_callback,
    destination = destination,
    teleport_on_fail = teleport_on_fail,
    is_retry = is_retry
  }

  local radius = 0.1
  if is_retry then
    radius = 3 -- 3 is the default if no radius is specified
  end

  local command =
  {
    type = defines.command.go_to_location,
    destination = destination,
    pathfind_flags = { allow_destroy_friendly_entities = allow_destroy_friendly_entities,
                       cache = false,
                       prefer_straight_paths = true },
    distraction = defines.distraction.none,
    radius = radius
  }

  command_callback.set_command(
    compi.get_entity(),
    command,
    -- If walking fails even though it should have succeeded, then try again,
    -- if that fails, just teleport.
    "walk_to_internal_callback"
  )
end

local walk_to_internal_callback = function (result)

  compi.get_entity().set_command
  {
    type = defines.command.stop,
    distraction = defines.distraction.none
  }

  local d = global.compilatron.walk_to_internal_retry_data
  global.compilatron.walk_to_internal_retry_data = nil
  if result ~= defines.behavior_result.success then
    if not d.is_retry then
      function_save.run(d.retry_callback, result)
      return
    elseif d.teleport_on_fail then
      compi.teleport_to(d.destination)
    end
  end

  -- arrived

  -- if we were asked to move somewhere while we were on our way to here, then start going to the other place
  if global.compilatron.next_destination_data ~= nil then
    local data = global.compilatron.next_destination_data
    global.compilatron.next_destination_data = nil

    walk_to_internal(data.destination,
                     data.teleport_on_fail,
                     data.allow_destroy_friendly_entities,
                     false,
                     data.retry_callback,
                     data.arrived_callback)
    return
  end

  compi.set_direction(global.compilatron.direction)


  if global.compilatron.wait_text then
    compi.say(global.compilatron.wait_text)
    global.compilatron.wait_text = nil
  end

  if global.compilatron.future_box then
    compi.indicate(global.compilatron.future_box)
  end
  global.compilatron.moving = false

  if d.arrived_callback then
    function_save.run(d.arrived_callback)
  end
end
command_callback.add_callback("walk_to_internal_callback", walk_to_internal_callback)

compi.set_wait_time_modifier = function(modifier)
  if type(tonumber(modifier))== 'number' and tonumber(modifier) > 0 and tonumber(modifier) < 100 then
    global.compilatron.wait_time_modifier = modifier
    return true
  end
  return false
end

compi.init = function(start_pos)
  game.map_settings.path_finder.max_steps_worked_per_tick = 500

  global.compilatron =
  {
    entity = nil,
    destroyed_entity_positions = {},
    destroyed_entity_ghosts = {},
    behavior = {},
    bubble = nil,
    indicator = nil,
    future_box = nil,
    now_text = nil,
    wait_text = nil,
    wait_time_modifier = 1,
    wait_position = start_pos,
    direction = defines.direction.southeast,
    moving = false
  }

  if start_pos then
    compi.respawn_compi(start_pos)
    assert(compi.get_entity().valid)
  end
  local compi_res_force = game.forces['compi_res_force'] or game.create_force("compi_res_force")
  compi_res_force.ghost_time_to_live = 99999
end

local compi_command = function(data)
  local params = util.split_whitespace(data.parameter)

  local player = game.players[data.player_index]
  if params[1] == 'come' then
    compi.walk_to(player.position)
    compi.say_later("???")
  elseif params[1] == 'wait' then
    compi.walk_to(global.compilatron.wait_position)
  elseif params[1] == 'show' then
    if player.selected then
      compi.stand_next_to(player.selected.selection_box, true, player.selected.name)
    end
  elseif params[1] == 'wait_time' then
    if compi.set_wait_time_modifier(params[2]) then
      player.print("Compilatron wait times are now multiplied by "..params[2])
    else
      player.print("This command requires a number between 0 and 100")
    end
  elseif params[1] == 'goto' then
    local x = tonumber(params[2])
    local y = tonumber(params[3])

    compi.walk_to({x, y})
  end
end
commands.add_command("compi","A series of commands to test Compilatron. Uses include: come, wait", compi_command)

compi.on_load = function()
end

compi.events = {
  [defines.events.on_entity_died] = on_entity_died,
  [defines.events.on_tick] = update,
  [defines.events.on_robot_built_entity] = on_robot_built_entity,
}

return compi
