local locations = require(mod_name .. '.lualib.locations')
local cutscene = require(mod_name .. '.lualib.cutscene')
local compi = require(mod_name .. '.lualib.compi')
local misc = require(mod_name .. '.lualib.misc')
local helper_story = require(mod_name .. '.lualib.npe.helper_story')
local math2d = require('math2d')

local compi_build_iron_cutscene = {}


compi_build_iron_cutscene.generate_cutscene = function()

  local build_iron_cutscene_nodes =
  {
    {
      name = 'compi-builds-iron-1',

      init = function()
        helper_story.set_paused(true)
        compi.walk_to(locations.get_pos("compi-build-iron-point"))
        global.demonstration_happened = true

        for _, player in pairs(game.connected_players) do

          local iron_area = locations.get_area('iron-patch-construction')
          local show_iron_area_params = misc.get_cutscene_show_area_parameters(iron_area, player.display_resolution, 10)
          local show_iron_area_params_zoomed = misc.get_cutscene_show_area_parameters(iron_area, player.display_resolution, 5)

          local compi_cutscene = {
            type = defines.controllers.cutscene,
            waypoints =
            {
              {
                position = player.position,
                zoom = show_iron_area_params.zoom,
                transition_time = 60 * 1,
                time_to_wait = 0,
                name = "zoom_out"
              },
              {
                position = show_iron_area_params.position,
                zoom = show_iron_area_params.zoom,
                transition_time = 60 * 2,
                time_to_wait = 60 * 0,
                name = "look_at_compi"
              },
              {
                position = show_iron_area_params.position,
                zoom = show_iron_area_params_zoomed.zoom,
                transition_time = 60 * 2,
                time_to_wait = 60 * 15,
              },
              {
                position = show_iron_area_params.position,
                zoom = show_iron_area_params.zoom,
                transition_time = 60 * 5,
                time_to_wait = 60 * 1,
                name = "watch_build"
              },
              {
                position = player.position,
                zoom = show_iron_area_params.zoom,
                transition_time = 60 * 2,
                time_to_wait = 60 * 5,
                name = "compi_finished",
              }
            },
            final_transition_time = 60 * 1
          }

          cutscene.play(compi_cutscene, { player })
        end
      end,

      condition = function()
        return cutscene.get_last_waypoint_reached_name() == "look_at_compi"
      end
    },

    {
      name = 'compi-builds-iron-2',
      init = function()
        compi.build_area('iron-patch-construction', {force = 'bystander'})
      end,

      condition = function()
        return cutscene.get_last_waypoint_reached_name() == "watch_build"
      end
    },

    {
      name = 'compi-builds-iron-3',
      init = function()
        compi.walk_to(math2d.bounding_box.get_centre(locations.get_area('compi-trigger-cutscene-crash')))
        compi.say_later({"compi.story-automation-complete"})
      end,
      condition = function()
        return cutscene.get_last_waypoint_reached_name() == "compi_finished"
      end,
      action = function()
        compi.say()
      end
    },

    {
      name = 'compi-builds-iron-cleanup',
      init = function()
        helper_story.set_paused(false)
        global.compilatron.reset = true
      end,
      condition = function()
        return not cutscene.is_any_cutscene_playing()
      end
    }
  }

  return build_iron_cutscene_nodes
end

return compi_build_iron_cutscene
