local cutscene = require(mod_name .. '.lualib.cutscene')
local locations = require(mod_name .. '.lualib.locations')
local colony_controller = require(mod_name .. '.lualib.colony_controller')
local eastern_cutscene_biters = require(mod_name .. '.lualib.npe.eastern_cutscene_biters')
local story = require("story_2")

local west_spawner_close = {}

 west_spawner_close.generate_cutscene = function()
  local story_nodes =
  {
    {
      name = "west_spawner_close-init",
      init = function()

        local west_spawner_biters = eastern_cutscene_biters.get_west_spawner_group(3)
        west_spawner_biters.set_command {
          type = defines.command.compound,
          structure_type = defines.compound_command.return_last,
          commands = {
            {
              type = defines.command.go_to_location,
              destination = locations.get_pos('cutscene-west-spawner'),
              radius = 1,
              distraction = defines.distraction.none,
            },
            {
              type = defines.command.wander,
              radius = 10,
              distraction = defines.distraction.none,
            }
          }
        }

        local zoom = 0.5

        local scene =
        {
          type=defines.controllers.cutscene,
          waypoints=
          {
            {
              position = main_player().position,
              zoom = zoom,
              transition_time = 60 * 1.5,
              time_to_wait = 0,
              name = "pan_to_player"
            },
            {
              position = "target-1",
              transition_time = 60 * 1,
              time_to_wait = 60 * 1,
              zoom = 1,
              name = "pan-to-target-1"
            },
            {
              position = "cutscene-west-spawner",
              transition_time = 60 * 2,
              time_to_wait = 60 * 5,
              zoom = 1,
              name = "pan-to-west-mid"
            },
            {
              position = main_player().position,
              transition_time = 60 * 1.5,
              time_to_wait = 0,
              zoom = zoom
            },
          },

          final_transition_time = 60 * 1.5
        }

        cutscene.play(scene)
      end,

      condition = function()
        return cutscene.get_last_waypoint_reached_name() == "pan-to-target-1"
      end
    },
    {
      name = "west_spawner_close-start_attack",
      init = function()
        local group = eastern_cutscene_biters.get_west_spawner_group(3)

        if group.valid then
          for _, biter in pairs(group.members) do
            colony_controller.send_to_area(biter,locations.get_area('colony-west-mid'),1200)
          end
        end
      end,
      condition = function()
        local biters_in_target = locations.get_main_surface().find_entities_filtered({
          name = 'small-biter',
          area = locations.get_area('colony-west-mid')
        })
        return #biters_in_target > 2 or
          cutscene.get_last_waypoint_reached_name() == "pan-to-west-mid"
      end,
    },
    {
      name = "west_spawner_close-wait-a-second",
      condition = function()
        return story.check_seconds_passed('main_story',1)
      end,
      action = function()
        colony_controller.register('colony-west-mid',1,0,'enemy')
        colony_controller.register('canyon-middle-rally',2,0,'enemy')
        local value = colony_controller.kill_all_units_in_colony('colony-west-mid')
        colony_controller.instant_spend_on_colony('colony-west-mid',value)
      end
    },
    {
      name = "west_spawner-final",
      condition = function()
        return not cutscene.is_any_cutscene_playing()
      end
    },
  }

  return story_nodes
end

return  west_spawner_close