--- Test Scenario: NPE V0.2
-- @script runtime
-- The script sets up and runs a scenario which should test
-- the basic functionality of the proposed tutorial level.

mod_name = "__base__"
campaign_debug_mode = script.is_game_in_debug_mode

local event_handler = require(mod_name .. ".lualib.event_handler")
local map_expand = require(mod_name .. ".lualib.map_expand")
local locations = require(mod_name .. ".lualib.locations")
local crashsite = require ("crash-site")
local forces = require("forces")
local pollution = require(mod_name .. ".lualib.pollution")
local storytable = require("storytable")
local story = require('story_2')
local quest_gui = require(mod_name .. ".lualib.quest_gui")
local popup = require(mod_name .. ".lualib.popup")
local compi = require(mod_name .. ".lualib.compi")
local helper = require(mod_name .. ".lualib.npe.helper_story")
local tracker = require(mod_name .. ".lualib.player_tracker")
local effects = require(mod_name .. ".lualib.effects")
local attacks = require(mod_name .. ".lualib.attacks")
local command_callback = require(mod_name .. ".lualib.command_callback")
local function_save = require(mod_name .. ".lualib.function_save")
local colony_controller = require(mod_name .. ".lualib.colony_controller")
local gui_helpers = require(mod_name .. '.lualib.npe.gui_helpers')
local cutscene = require(mod_name .. ".lualib.cutscene")
local misc = require(mod_name .. ".lualib.misc")
local scenario_events = require("scenario-events")
local waves = require("waves")
local eastern_cutscene_biters = require(mod_name .. ".lualib.npe.eastern_cutscene_biters")
local technology_manager = require(mod_name .. '.lualib.technology_manager')
local campaigns_version = require(mod_name .. '.lualib.campaigns_version')
local tech_levels = require('tech_levels')
local soft_death = require(mod_name .. '.lualib.soft_death')
local util = require('util')
local position_data = require('position_data')
require('scenario-commands')

-- Unfortunately, we can currently require lua modules in a few different ways, eg one relative path vs another,
-- "in-mod" absolute paths, etc. When we do this, the engine sees them as different files. For now, we will workaround
-- this by using absolute in-mod paths, and this code block asserts that we've done this correctly.
local module_name_mapping = {}
for key, _ in pairs(package.loaded) do
  local split = util.split(key, '.')
  local split_name = split[#split]
  assert(split_name)

  if module_name_mapping[split_name] then
    if module_name_mapping[split_name] ~= key then
      error("Module imported twice with two different paths: " .. key .. ", " .. module_name_mapping[split_name])
    end
  else
    module_name_mapping[split_name] = key
  end
end

main_player = function()
  for _, player in pairs(game.forces['player'].players) do
    if player.controller_type == defines.controllers.character or player.controller_type == defines.controllers.cutscene then

      return player
    end
  end
  return nil
end

active_player = function()
  for _, player in pairs(game.forces['player'].players) do
    if player.controller_type == defines.controllers.character or player.controller_type == defines.controllers.cutscene then
      return player
    end
  end
  return nil
end

local on_player_join = function(event)
  game.players[event.player_index].disable_recipe_groups()
  game.players[event.player_index].disable_recipe_subgroups()
end

local setup_events = function()
  local main_events =
  {
    [defines.events.on_player_joined_game] = on_player_join,
  }

  local event_receivers =
  {
    main_events,
    map_expand.events,
    forces.events,
    storytable.events,
    popup.events,
    compi.events,
    helper.events,
    effects.events,
    tracker.events,
    command_callback.events,
    gui_helpers.events,
    cutscene.events,
    misc.events,
    colony_controller.events,
    quest_gui.events,
    scenario_events.events,
    soft_death.events,
    function_save.events
  }

  event_handler.setup_event_handling(event_receivers)
end

local _compi_build_areas = nil
local get_compi_build_areas = function()
  if _compi_build_areas == nil then
    _compi_build_areas =
    {
      locations.get_area('iron-patch-construction'),
      locations.get_area('gear-feed-construction'),
      locations.get_area('lab-feed-construction'),
    }
  end

  return _compi_build_areas
end

local on_game_created_or_loaded = function()
  if campaign_debug_mode then
    print("ON_LOAD: control")
  end

  if not global.CAMPAIGNS_VERSION or global.CAMPAIGNS_VERSION < 11 then
    return
  end

  locations.on_load()
  map_expand.on_load(get_compi_build_areas())
  storytable.on_load()
  helper.on_load()
  compi.on_load()
  cutscene.on_load()
  popup.on_load()
  misc.on_load()
  quest_gui.on_load()
  colony_controller.on_load()
  attacks.on_load(waves)
  eastern_cutscene_biters.on_load()
  command_callback.on_load()
  soft_death.on_load()
  technology_manager.on_load(tech_levels, {})
  function_save.on_load()

  if global.CAMPAIGNS_VERSION == campaigns_version.current_version then
    locations.verify_data(position_data.points, position_data.areas)
  end

  if global.scenario_started then
    setup_events()
  end
end
script.on_load(function()
  on_game_created_or_loaded()
end)

local on_game_created_from_scenario = function()
  global.CAMPAIGNS_VERSION = campaigns_version.current_version
  locations.init('nauvis','template')
  map_expand.init(get_compi_build_areas())
  crashsite.init()
  forces.init()
  pollution.init(0,0,0.000006,0.1,true,15,4)
  storytable.init()
  popup.init()
  command_callback.init()
  compi.init(locations.get_pos('compi-wait-crash'))
  helper.init()
  cutscene.init()
  tracker.init()
  gui_helpers.init()
  popup.init()
  misc.init()
  quest_gui.init()
  colony_controller.init()
  eastern_cutscene_biters.init()
  technology_manager.init(tech_levels, {})
  attacks.init()
  soft_death.init(300)
  function_save.init() -- this must stay last, as it ensures other init funcs can register functions
  on_game_created_or_loaded()
  global.playthrough_number = nil

   -- disallow wire dragging in this scenario
  game.permissions.get_group("Default").set_allows_action(defines.input_action.wire_dragging, false)
  game.disable_tips_and_tricks()

  locations.verify_data(position_data.points, position_data.areas)
end

script.on_event(defines.events.on_game_created_from_scenario, function()
  script.on_event(defines.events.on_game_created_from_scenario, nil)
  setup_events()
  on_game_created_from_scenario()
  global.scenario_started = true
end)

local is_global_empty = function()
  return next(global) == nil
end

local show_migrate_too_old = function()
  local heading = {'campaign-migrated-too-old-message.heading'}
  local blocks = {{'campaign-migrated-too-old-message.text'}}

  if main_player().gui.center.disclaimer then
    main_player().gui.center.disclaimer.destroy()
  end
  local window = main_player().gui.center.add({
    type='frame',
    name='disclaimer',
    direction='vertical',
    style = 'dialog_frame_no_header_filler',
    caption = heading,
    })
  local holder = window.add({
    type='table',
    column_count=1,
    })

  for _, block in pairs(blocks) do

    local msg = holder.add({
      type='label',
      caption=block
      })
    msg.style.single_line = false
    msg.style.maximal_width = 300
    msg.style.bottom_padding = 10

  end
  local horizontal_flow = window.add({
    type='flow',
    direction='horizontal',
    })
  local thing = horizontal_flow.add({
    type = 'flow',
    direction='horizontal',
    style= 'quest_item_icons_wrapper'
  })
  thing.style.horizontally_stretchable = true
  thing.style.maximal_width = 300
  horizontal_flow.add({
    type='button',
    name='close-text-window-quit-game',
    style='confirm_button',
    caption={'text-window.continue-button'},
    })

  game.tick_paused = true
end

script.on_event(defines.events.on_gui_click, function(event)
    if event.element then
      if event.element.name == 'close-text-window-quit-game' then
         game.set_game_state({game_finished=true, player_won=true, can_continue=false})
      end
    end
  end)


local on_configuration_changed = function()
  game.reload_script()
  if is_global_empty() then return end

  if global.CAMPAIGNS_VERSION == nil then
    global.CAMPAIGNS_VERSION = 1
  end

  if global.CAMPAIGNS_VERSION < 11 then
    -- This will just exit the game
   show_migrate_too_old()
    return
  end

  -- Migrations can cause this to be reset
  game.forces.compi_res_force.ghost_time_to_live = 99999

  if global.CAMPAIGN_MIGRATIONS_APPLIED == nil then
    global.CAMPAIGN_MIGRATIONS_APPLIED = {}
  end

  technology_manager.reset()

  if global.CAMPAIGNS_VERSION ~= campaigns_version.current_version then

    quest_gui.migrate()
    technology_manager.migrate()

    table.insert(global.CAMPAIGN_MIGRATIONS_APPLIED, {
      global.CAMPAIGNS_VERSION,
      campaigns_version.current_version,
      story.get_current_node_name('main_story')
    })

    global.CAMPAIGNS_VERSION = campaigns_version.current_version
  end
end
script.on_configuration_changed(on_configuration_changed)
